#define SerialMon Serial
#define SerialAT Serial1
#define TINY_GSM_MODEM_ESP8266

#include <Adafruit_CircuitPlayground.h>
#include <FiniteStateMachine.h>
#include "Flasher.h"
#include <TinyGsmClient.h>
#include <PubSubClient.h>

// WiFi connection credentials
const char wifiSSID[] = "Your_WiFi_SSID";
const char wifiPass[] = "Your_WiFi_Password";

// MQTT details
const char MQTT_CLIENTID[] = __DATE__ __TIME__;
const char* broker = "Your_MQTT_Broker_IP";
char lightTopic[] = "light";
char temperatureTopic[] = "temperature";
char statusTopic[] = "status";
unsigned pushInterval = 10 * (60 * 1000);             // The interval between broker pushes (milliseconds) - currently 10 minutes
unsigned long pushTimeout = 24 * (60 * 1000 * 60);    // The time to stop the sensors fetching (millisecons) - currently 24 hours

// Network enviorment
TinyGsm modem(SerialAT);
TinyGsmClient client(modem);
PubSubClient mqtt(client);
uint32_t lastReconnectAttempt = 0;

// Variable decleration
char valueBuffer[50];
boolean push = false;
boolean started = false;
long lastPublished = 0;
Flasher flasher(100, 40, 180, 1000);

#include "Connections.h"

// Start collect and push data to the MQTT broker
void onPushStart() {
  
    mqttLoop();
    
    Serial.println("Start Pushing");
    started = true;
    push = true;
    mqtt.publish(statusTopic, "START");
    pushTimeout += millis();
        
}

// State machine states
State pushState = State(onPushStart, pushUpdate, onPushEnd);
State doneState = State(onDoneStart, doneUpdate, NULL);

// State machine
FSM stateMachine = FSM(pushState);


// Setup function
void setup() {
  
    CircuitPlayground.begin();
    randomSeed(analogRead(A1) ^ analogRead(A2) ^ analogRead(A3));
    
    initModem();
    connectToWifi();
    mqttConnect(false);

}

// Loop function
void loop() {

    stateMachine.update();
    
}

// Collect sensors data and push to the MQTT broker
void pushUpdate() {
  
    mqttLoop();
    
    if (push) {
      
        flasher.Update();
        if (millis() - lastPublished > pushInterval) {
          
            Serial.println("Publishing Light");
            float fLight = CircuitPlayground.lightSensor();
            mqttPublish(String(fLight), lightTopic);
        
            Serial.println("Publishing Temperature");
            float fTemp = CircuitPlayground.temperature();
            mqttPublish(String(fTemp), temperatureTopic);
            
            lastPublished = millis();
            
        }
      
        if (millis() > pushTimeout) {
            stateMachine.transitionTo(doneState);
        }
    }
    else {
        flasher.Destroy();
    }
}

// Done to collect and push data to the MQTT broker
void onPushEnd() {
    Serial.print("Done Pushing");
    push = false;
    mqtt.publish(statusTopic, "DONE");
}

// Notify the user that the collection is done
void onDoneStart() {
    for (int i = 0; i < 3; i++) {
        
        colorPixels(random(255), random(255), random(255));
        CircuitPlayground.playTone(500, 500);
        
    }
    colorPixels(random(255), random(255), random(255));
    CircuitPlayground.playTone(700, 500);
    delay(1000);
    
}

// Done visual message
void doneUpdate() {

    colorPixels(random(255), random(255), random(255));
    delay(1500);
    
}
